function etmToGpopsProcessing
%% Set variables
clear all;

% NOTE (Nathan): For some reason this has the right leg first, whereas the
% actual EtM code has the left leg first.

hpf = 40; % high pass filter cutoff for the EMG data
lpfVar = 3.5;%7/1.8; % low pass filter cutoff for the EMG data REMOVED
nPtsLong = 141; % number of sample points in gait cycle with padded beginnings and ends
nPtsShort = 101; % number of sample points in gait cycle without pading
trialsStep = 1;
nTrials = 1;

% Specify number of muscles and EMG signals for each muscle
nEmg = 14; % Number of EMG signals recorded
nMuscEmg = 31; % Number of muscles with EMG available
nMuscReconst = 5; % Number of muscles which will have activations reconstructed using synergies
nReconstDup = 1; % Number of reconstructed muscle activations which are duplicates of other reconstructed activations
nMuscTotal = nMuscEmg + nMuscReconst; % Total number of muscles in the model
haveEmg = 1:31; % Which mucles (out of total) have EMG available
nSyn = 5; % Number of synergies to use

% Subject height and mass for max muscle force calculation
height = 1.74; % m
mass = 78.86; % kg

coefsScaleFactors = 0.1; %coefsScaleFactors/10;

%% Load trial data
load('Subject1_v4pt3_etmData_bothLegs_replacedRMGas_outliersRemoved.mat');
trialIndex_l = 79; % Trial04_cycle_12 OLD: 84 
trialIndex_r = 82; % Trial04_cycle_12 OLD: 91
synergyTrialIndex_l = trialIndex_l - 75 + 20; % -75 is because we skipped a lot of trials to get to fast gait speed, +20 is for because the first 20 are from slow speed
synergyTrialIndex_r = trialIndex_r - 75 + 20;
TrialName = etmData_l(trialIndex_l).TrialName;

% Extract Joint Coordinates
jAngles = [etmData_r(trialIndex_r).JointAngles,etmData_l(trialIndex_l).JointAngles];
jVels =  [etmData_r(trialIndex_r).JointVelocities,etmData_l(trialIndex_l).JointVelocities];
time = etmData_r(trialIndex_r).Time;
% [coords, coordinateLabels, Time] = readMotFile(motionFileName);
% Time = Time(1:141);
% TimeLongest = Time;
% 
% % Take desired DOFs from coordinates (left leg only, sagittal plane)
% coords = coords(1:141,[7 8 9 10 12 13 15 16 17 18 20 21]);
% coordNames = {'Right Hip FE' 'Right Hip AA' 'Right Knee FE' 'Right Ankle FE' 'Right Subtalar IE' 'Right Hip IE' ...
%     'Left Hip FE' 'Left Hip AA' 'Left Knee FE' 'Left Ankle FE' 'Left Subtalar IE' 'Left Hip IE'};
% JVels = coords;
% for i = 1:12
%     [coords(:,i), JVels(:,i)] = SplineSmooth(TimeLongest(1:length(coords)), coords(:,i), .9999);
% end
% 
% JAngles = coords*pi/180;
% JVels = JVels*pi/180;

% Load inverse dynamics torques
idLoads = [etmData_r(trialIndex_r).InverseDynamicsLoads,etmData_l(trialIndex_l).InverseDynamicsLoads];
% IDnames = {'Hip FE Moment' 'Hip AA Moment' 'Knee FE Moment' 'Ankle FE Moment' 'Subtalar IE Moment'};
% [IDloads, IDLabels, TimeID] = readMotFile(IDFileName);
% IDloads = IDloads(:,[7 8 10 12 13 15 16 18 20 21]);


% Load emg data
emgData = [etmData_r(trialIndex_r).EMG,etmData_l(trialIndex_l).EMG];
% [emgData, emgLabels, TimeEMG] = readMotFile(emgFileName);
% emgLabels = emgLabels(2:33);
% emgData = emgData(:,1:32);


% nMusc = length([2:71]);


%% copy from model optimization

% Load initial guesses for the muscle fits
load Subject1_v4pt1_MuscleFits_lhs.mat

% Append all individual muscle coefficients into a single dimension array form
coefsFit = MuscleFits.Coefs;
coefsOrig = [];
for i = 1:nMuscTotal
    coefsOrig = [coefsOrig; coefsFit{i}];
end

% Number of coefficients used in the muscle geometry fits
nCoefsFit = length(coefsOrig);

% define muscle reference matrix that defines which joint(s) a muscle
% actuates
muscRef = zeros(1,nMuscTotal);
muscRef([1:16,32:33]) = 1; % Hip FE and AA
muscRef([17:19 21]) = 2; % Hip FE and AA and Knee FE
muscRef([20 22:24]) = 3; % Knee FE
muscRef([25:26]) = 4; % Knee FE, Ankle and Subtalar
muscRef([27:31,34:36]) = 5; % Ankle and Subtalar

load Subject1_v4pt3_scaledLmoLts.mat

optParams.lmo = lmo;
optParams.lts = lts;

% Preallocate memory
emg = zeros(nPtsLong,2*nEmg,nTrials);
timeEmg = time;
for i = 1
    
    j = ceil(i/trialsStep);
    
    % determine time length from heel strike to heel strike (not including
    % padding on beginning and end of cycle)
    tf = (timeEmg(end,j)-timeEmg(1,j))*(nPtsShort/nPtsLong);
    lpf = lpfVar/tf;
    
    % catch a common step parsing error in preprocessing
    if tf >10
        keyboard
    end
    
    % process the EMG data
    emgTrial = processEMG(emgData, 1/1000, hpf, lpf);
    
    % find original final time of trial
    tf = (length(emgTrial)-1)/1000;
    
    % create normalized emg signals time vectors
    timeEmgtrial = 0:(1/1000):tf;
    timeEmg(:,i) = linspace(0,tf,nPtsLong);
    % Normalize to nptsLong time frames
    emg(:,:,j) = spline(timeEmgtrial, emgTrial', timeEmg(:,i))';
    
end

% Duplicate EMG signals for similar muscles
emg = emg(:,[[6*ones(1,6) 7*ones(1,3) 9*ones(1,6) 8 10*ones(1,2) 5*ones(1,2) 4 11 3*ones(1,2) 2 12 1 13*ones(1,3) 14] [6*ones(1,6) 7*ones(1,3) 9*ones(1,6) 8 10*ones(1,2) 5*ones(1,2) 4 11 3*ones(1,2) 2 12 1 13*ones(1,3) 14]+14],:);

% Load EMG scale factors determined for all trials
load EMGmaxima_l.mat
load EMGmaxima_r.mat

% split EMG into left and right side
emg_r = emg(:,1:nMuscEmg);
emg_l = emg(:,(nMuscEmg+1):end);

% Remove EMG offset for each trial
emgMins_l = min(emg_l(21:121,:,:),[],1);
emg_l = emg_l-emgMins_l(ones(141,1),:,:);
emgMins_r = min(emg_r(21:121,:,:),[],1);
emg_r = emg_r-emgMins_r(ones(141,1),:,:);

for i = 1:nMuscEmg
    emg_l(:,i,:) = (emg_l(:,i,:)-EMGmin_l(i))/EMGmax_l(i);
    emg_r(:,i,:) = (emg_r(:,i,:)-EMGmin_r(i))/EMGmax_r(i);
end

emg = cat(2,emg_r,emg_l);

optParams.emg = permute(emg, [1 3 2]);

emgSplines = cell(1,nTrials);

emgSplines{1} = spline(time(1:4:end,j),emg(1:4:end,1:nMuscEmg,j)'); % Use every other frame to improve speed when evaluating spline
emgSplines{2} = spline(time(1:4:end,j),emg(1:4:end,(nMuscEmg+1):(2*nMuscEmg),j)');

optParams.emgSplines = emgSplines;

% Load optimization parameter values
load params_Subject1_hipWeighted_v4pt3_bothSpeeds.mat
load synergies_Subject1_hipWeighted_v4pt3_bothSpeeds.mat

reconstSynWeights = params(1:(2*nSyn*(nMuscReconst-nReconstDup)));
index = 2*nSyn*(nMuscReconst-nReconstDup);
tDelay = params(1,index+1:index+2)/10;
tDelay = [tDelay(2),tDelay(1)]; % This is because of switching from left leg first to right leg first
index = index + 2;
tAct = params(1,index+1:index+nMuscEmg)/100;
index = index + nMuscEmg;
tDeact = 4*tAct;
aNonlinOpt = params(1,index+1:index+nMuscEmg);
index = index + nMuscEmg;
emgScaleOpt = params(1,(index+1):(index+2*nMuscEmg));
emgScaleOpt = [emgScaleOpt((nMuscEmg+1):end),emgScaleOpt(1:nMuscEmg)]; % This is because of switching from left leg first to right leg first
index = index + 2*nMuscEmg;
vMaxFactor = 10;
lmoScaleOpt = params(1,(index+1):(index+nMuscTotal));
index = index + nMuscTotal;
ltsScaleOpt = params(1,(index+1):(index+nMuscTotal));
index = index + nMuscTotal;
coefsOffsetOpt = params(1,(index+1):end).*coefsScaleFactors;

coefs = coefsOrig+coefsOffsetOpt';
lts = optParams.lts.*ltsScaleOpt;
lmo = optParams.lmo.*lmoScaleOpt;


optParams.vMaxFactor = vMaxFactor;
optParams.lmoScale = lmoScaleOpt;
optParams.ltsScale = ltsScaleOpt;
optParams.emgScale = emgScaleOpt;
optParams.tDelay = tDelay;
optParams.tAct = tAct;
optParams.tDeact = tDeact;
optParams.aNonlin = aNonlinOpt;

optParams.nMuscTotal = nMuscTotal;
optParams.nMuscEmg = nMuscEmg;
optParams.nEmg = nEmg;
optParams.nTrials = nTrials;
optParams.time = time;
optParams.onesCol = ones(101,1);
optParams.nptsShort = 101;
optParams.nptsLong = 141;
optParams.sampleStep = 1;
optParams.nTrials_l = 1;
optParams.nTrials_r = 1;

[a, excitations] = EMGtoAct(optParams);

synWeightsTemp = reconstSynWeights(1,1:(nSyn*(nMuscReconst-nReconstDup)));
reconstSynWeights_l = reshape(synWeightsTemp,[nSyn,(nMuscReconst-nReconstDup)]);
synWeightsTemp = reconstSynWeights(1,(nSyn*(nMuscReconst-nReconstDup)+1):(2*nSyn*(nMuscReconst-nReconstDup)));
reconstSynWeights_r = reshape(synWeightsTemp,[nSyn,(nMuscReconst-nReconstDup)]);

unknownActs_l = synergyCommandsNorm_l(((synergyTrialIndex_l-1)*nPtsShort + 1):(synergyTrialIndex_l*nPtsShort),:)*reconstSynWeights_l;
unknownActs_r = synergyCommandsNorm_r(((synergyTrialIndex_r-1)*nPtsShort + 1):(synergyTrialIndex_r*nPtsShort),:)*reconstSynWeights_r;
unknownActs = [unknownActs_r,unknownActs_l];

a = [a(:,1:nMuscEmg),unknownActs_r(:,1),unknownActs_r,a(:,(nMuscEmg+1):(2*nMuscEmg)),unknownActs_l(:,1),unknownActs_l]; % NOTE: this part is specific to the muscles and EMG you have


synergy.weights_l = [synergyWeightsNorm_l,reconstSynWeights_l(:,1),reconstSynWeights_l];
synergy.weights_r = [synergyWeightsNorm_r,reconstSynWeights_r(:,1),reconstSynWeights_r];
synergy.commandsExp_l = synergyCommandsNorm_l(((synergyTrialIndex_l-1)*nPtsShort + 1):(synergyTrialIndex_l*nPtsShort),:);
synergy.commandsExp_r = synergyCommandsNorm_r(((synergyTrialIndex_r-1)*nPtsShort + 1):(synergyTrialIndex_r*nPtsShort),:);



%% Format EtM results for output
k = 1;
for i = 1:nMuscTotal
    if muscRef(i) == 1
        coefsOpt{i} = coefs(k:k+18);
        k = k+19;
    elseif muscRef(i) == 2
        coefsOpt{i} = coefs(k:k+21);
        k = k+22;
    elseif muscRef(i) == 3
        coefsOpt{i} = coefs(k:k+3);
        k = k+4;
    elseif muscRef(i) == 4
        coefsOpt{i} = coefs(k:k+12);
        k = k+13;
    elseif muscRef(i) == 5
        coefsOpt{i} = coefs(k:k+9);
        k = k+10;
    end
end

alpha = [6.1 7.1 13.8 11.9 14.7 22.2 21.1 21.9 22.8 20.5 20.5 20.5 10.0 0 1.0 3.0 15.1 12.9 11.6 12.3 13.9 29.6 18.4 4.5 12.0 9.9 9.6 11.5 14.1 13.0 28.3 14.3 10.7 13.7 10.8 13.6]*pi/180;

sigma = 610e3;
legVol = (47*height*mass+1285)/100^3;
volFraction = [1.47 2.26...
    1.97 1.97 1.97 1.97 ...
    3.52 5.02 3.39 ...
    1.82 1.27 1.45...
    0.45 0.48 0.54...
    0.89 3.46 2.60 2.92 1.40 3.79 6.06 11.66 3.84 2.11 3.62 1.91...
    0.53 1.14 0.16 6.21...
    2.48 3.80 1.49 0.97 0.43];
volFraction = volFraction/sum(volFraction);
volMusc = legVol*volFraction;

fMax = (volMusc./lmo)*sigma;

% double parameters to account for each leg seperately
coefs = [coefsOpt coefsOpt];
lmo = [lmo lmo];
lts = [lts lts];
alpha = [alpha alpha];
fMax = [fMax fMax];
aNonlin = [aNonlinOpt aNonlinOpt];
pennAngle = alpha;

% a = a(:,[36:70 1:35]);
% save act0pt5.mat a

save Subject1_v4pt1_EtM_params_trial04Cycle12.mat emg a excitations lmo lts coefs pennAngle fMax tDelay tAct tDeact aNonlin vMaxFactor emgScaleOpt synergy
save Subject1_v4pt1_EtM_data_trial04Cycle12.mat emg a excitations lmo lts coefs pennAngle fMax tDelay tAct tDeact aNonlin vMaxFactor emgScaleOpt synergy idLoads jAngles jVels time muscRef
end

%% Other Functions

function [a, excitations] = EMGtoAct(optParams)

nMuscEmg = optParams.nMuscEmg;
nTrials = optParams.nTrials;
time = optParams.time;
tDelay = optParams.tDelay;
emgSplines = optParams.emgSplines;
tAct(1,:) = [optParams.tAct optParams.tAct];
tDeact(1,:) = [optParams.tDeact optParams.tDeact];
aNonlin = [optParams.aNonlin optParams.aNonlin];
emgScale = optParams.emgScale;
onesCol = optParams.onesCol;
nptsShort = optParams.nptsShort;
nptsLong = optParams.nptsLong;
sampleStep = optParams.sampleStep;

% subplot(1,5,1)
% hold on
% plot(0:100,optParams.EMG(21:121,50,24),'k','LineWidth',3)
% set(gca,'FontSize',24,'xtick',[0 100],'ytick',[0 0.5 1])
% xlabel('% Gait Cycle')
% title('EMG Envelope')
% axis([0 100 0 1])

interpStep = 1;
nptsInterp = interpStep*(nptsLong-1)+1;

timeInterp = linspace(0,1,(size(time,1)-1)*interpStep+1)';

emg(:,1:nMuscEmg) = ppval((time(end,1)-time(1,1))*timeInterp+time(1,1)-tDelay(1),emgSplines{1})';
emg(:,(nMuscEmg+1):(2*nMuscEmg)) = ppval((time(end,1)-time(1,1))*timeInterp+time(1,1)-tDelay(2),emgSplines{2})';

% subplot(1,5,2)
% hold on
% plot(0:100,EMG(21:121,50,24),'k','LineWidth',3)
% set(gca,'FontSize',24,'xtick',[0 100],'ytick',[])
% xlabel('% Gait Cycle')
% title('Shifted EMG')
% axis([0 100 0 1])

emg = emg.*emgScale(ones(nptsInterp,1),:);

% subplot(1,5,3)
% hold on
% plot(0:100,EMG(21:121,50,24),'k','LineWidth',3)
% set(gca,'FontSize',24,'xtick',[0 100],'ytick',[])
% xlabel('% Gait Cycle')
% title('Scaled EMG')
% axis([0 100 0 1])

excitations = emg((nptsInterp-1)/7+1:interpStep:nptsInterp-(nptsInterp-1)/7,:,:);

b2(1,:) = 1./tDeact;
b1(1,:) = 1./tAct-b2;
b3 = b1(ones(nptsLong,1),:).*emg + b2(ones(nptsLong,1),:);
dt(1,:,1) = mean(diff(time));
b4 = 2*dt(ones(nptsLong,1),ones(2*nMuscEmg,1)).*b3;

emg = b4.*emg;

a = zeros(size(emg));
for j = 3:nptsLong
    a(j,:,:) = (emg(j,:) + 4.*a(j-1,:) - a(j-2,:))./(b4(j,:) + 3);
end

% dt(1,1) = mean(diff(Time))/interpStep;
% c2 = 1./(tdeact);
% c1 = (1./(tact))-c2;
% dt = dt(:,ones(2*nMusc,1));
% b = 10;
% 
% % % Euler integration
% % a = EMG;
% % for i = 2:1:nptsInterp
% %     dadt = (c1.*EMG(i-1,:,:).*(0.5.*tanh(b.*(EMG(i-1,:,:)-a(i-1,:,:)))+0.5)+c2).*(EMG(i-1,:,:)-a(i-1,:,:));                    
% %     a(i,:,:) = a(i-1,:,:)+dt.*dadt;
% % end
% % toc
% % 
% % a = a((interpStep*(21-1)/SampleStep+1):interpStep:(interpStep*(121-1)/SampleStep+1),:,:);
% % aE = a;
% 
% % He equation
% actEqn = @(emgVal,actVal) (c1.*emgVal+c2).*(emgVal-actVal);
% 
% % Fregly equation
% actEqn = @(emgVal,actVal) (c1.*emgVal.*(0.5.*tanh(b.*(emgVal-actVal))+0.5)+c2).*(emgVal-actVal);
% 
% % Runge Kutta integration
% a = EMG;
% for i = 3:nptsInterp
%     
%     k1 = (c1.*EMG(i-2,:).*(0.5.*tanh(b.*(EMG(i-2,:)-a(i-2,:)))+0.5)+c2).*(EMG(i-2,:)-a(i-2,:));                    
%     atemp = a(i-2,:)+dt.*k1;
%     k2 = (c1.*EMG(i-1,:).*(0.5.*tanh(b.*(EMG(i-1,:)-atemp))+0.5)+c2).*(EMG(i-1,:)-atemp);                    
%     atemp = a(i-2,:)+dt.*k2;
%     k3 = (c1.*EMG(i-1,:).*(0.5.*tanh(b.*(EMG(i-1,:)-atemp))+0.5)+c2).*(EMG(i-1,:)-atemp);                    
%     atemp = a(i-2,:)+2*dt.*k3;
%     k4 = (c1.*EMG(i,:).*(0.5.*tanh(b.*(EMG(i,:)-atemp))+0.5)+c2).*(EMG(i,:)-atemp);
%     
%     a(i,:) = a(i-2,:)+dt/3.*(k1+2*k2+2*k3+k4);
% end

a = a((interpStep*(21-1)/sampleStep+1):interpStep:(interpStep*(121-1)/sampleStep+1),:);
% aRK = a;
% 
% close all
% plot(EMG(:,3,1),'k')
% hold on
% plot(aE(:,45,33),'b')
% hold on
% plot(aRK(:,45,33),'r')

% subplot(1,5,4)
% hold on
% Anonlin(24) = 0.15;
% plot(0:100,a(:,50,24),'k','LineWidth',3)
% set(gca,'FontSize',24,'xtick',[0 100],'ytick',[])
% xlabel('% Gait Cycle')
% title('Activation')
% axis([0 100 0 1])
% set(gca,'FontSize',24)

a = reshape(a, nptsShort*nTrials, 2*nMuscEmg);
a(a<0) = 0;

nonlincoefs = [29.280183270562596   4.107869238218326 1.000004740962477...
    -7.623282868703527 17.227022969058535   0.884220539986325];

a = (1-aNonlin(onesCol,:)).*a+(aNonlin(onesCol,:)).*...
    (nonlincoefs(4)./(nonlincoefs(1)*(a+nonlincoefs(6)).^nonlincoefs(5)+nonlincoefs(2))+nonlincoefs(3));


% a = reshape(a,nptsShort, nTrials, nMusc);
% subplot(1,5,5)
% hold on
% plot(0:100,a(:,50,24),'k','LineWidth',3)
% set(gca,'FontSize',24,'xtick',[0 100],'ytick',[])
% xlabel('% Gait Cycle')
% title('Nonlinear Activation')
% axis([0 100 0 1])
% set(gca,'FontSize',24)
end

function emgFiltered = processEMG(emgData, dt, HPcutoff, LPcutoff)
% Function that process EMG data according the the process described by
% Lloyd and Besier (2003)
% Inputs:
%       emgData is a M x N matrix of unfiltered EMG data where M is the
%           number of time frames and N is the number of muscles
%       dt is the time step used in the EMG data
%       HPcutoff is the cutoff frequency for the high pass filter
%       LPcutoff is the cutoff frequency for the low pass filter
% Outputs:
%       emgFiltered is the filtered EMG data

SampRate = 1/mean(dt);

% High pass filter the data
degree = 4;
[b,a] = butter(degree, 2*HPcutoff/SampRate,'high');
% [sos,g] = zp2sos(z,p,k);      % Convert to SOS(second-order-sections) form
EMG_HP = filtfilt(b,a,emgData);

% Demean
EMG_DM = EMG_HP-ones(size(EMG_HP,1),1)*mean(EMG_HP);

% Rectify
EMG_RF = abs(EMG_DM);


[b,a] = butter(degree,2*LPcutoff/SampRate);

EMG_LP = filtfilt(b,a,EMG_RF);
emgFiltered = EMG_LP;


% Remove any negative EMG values that may still exist
emgFiltered(emgFiltered<0) = 0;
end

function [Data, ColumnLabels, Time] = readMotFile(inFile)

%open file
fid = fopen(inFile);

%skip initial lines
line = 'asdf';
while ~strcmp(upper(line),'ENDHEADER')
    line = fgetl(fid);
end

% Find the labels on the columns
ColumnLabels = fgetl(fid);
ColumnLabels = textscan(ColumnLabels, '%t');
ColumnLabels = ColumnLabels{1}';

% Read numeric data
Data = fscanf(fid, '%f');

% Find length of data
nColumns = length(ColumnLabels);
nRows = length(Data)/nColumns;

% Reshape data into matrix
Data = reshape(Data, nColumns, nRows)';

% Extract time vector
Time = Data(:,1);
Data(:,1) = [];

fclose(fid);
end